﻿var portConnected = null;
var emulatorConnected = false;
var xslPorts = loadXML("ports.xsl");
var xmlPids = loadXML("pids.xml");
var pidDefaults;
var groups = xmlPids.getElementsByTagName("group");
var ignition = true;

transformXML(xmlPids, loadXML("tabs.xsl"), "tabs");
transformXML(xmlPids, loadXML("knobs.xsl"), "panel");

function onLoad() {
    var xmlDefaults = loadXML("defaults.xml");
    if (xmlDefaults)
        pidDefaults = xmlDefaults.getElementsByTagName("param");
    //self.setTimeout(listSerialPorts, 1000);
}

function listSerialPorts() {
    transformXML(loadXML("/serial"), xslPorts, "ports");
}

function chooseFile() {
	var path = httpGet('/choosefile');
	if (path) {
		filepath = path;
		LoadCode(filepath);
		return path;
	}
	return null;
}

function findEmulator() {
    var xmlSerialPorts = loadXML("/serial");
    var ports = xmlSerialPorts.getElementsByTagName("port");
    var portFound = null;
    for (var n = ports.length - 1; n >= 0; n--) {
        portFound = ports[n].getAttribute("name");
		var success;
		if (portConnected = SerialOpen(portFound, 38400)) {
		    success = sendCommand("ATINF0") || sendCommand("ATINF0");
		}
        SerialClose();
		portConnected = false;
        if (success) {
            if (connect(portFound))
                break;
        }
        portFound = null;
    }
    if (!portFound) {
        alert("Freematics OBD-II Emulator is not connected.\nPlease connect it to the PC via USB cable.\n\nFor product information, please visit http://freematics.com");
        disconnect();
    }
}

function autoConnect()
{
    if (portConnected) {
        disconnect();
        return;
    }
	emulatorConnected = false;
    document.getElementById("button_conn").innerHTML = "Connecting...";
    self.setTimeout(findEmulator, 10);
}

function init() {
    if (!sendCommand("ATE0") && !sendCommand("ATE0")) {
        alert("Error occurred during initialization.");
        disconnect();
        return false;
    }

    document.getElementById("status").className = "top_connected";
	document.getElementById("button_conn").innerHTML = "Disconnect";
	document.getElementById("button_ignite").disabled = false;

    /*
    var result = SerialWrite("ATZ\r", "%0D", 10000);
    if (!result) {
        alert("Error occurred during initialization.");
        disconnect();
        return;
    }
    */
    setDefaultPidValue(0);	
}

function connect(port) {
	if (!SerialOpen(port, 38400)) {
		alert("Unable to connect via " + port);
		return false;
	}

	portConnected = port;
	self.setTimeout(init, 10);
	return true;
}

function disconnect() {
	SerialClose();
	portConnected = null;
	document.getElementById("status").className = "top";
	document.getElementById("button_conn").innerHTML = "Connect";
	document.getElementById("button_ignite").disabled = true;
}

function sendCommand(cmd) {
    if (!portConnected) {
        return false;
    }
    return SerialWrite(cmd + "\r", "OK%0D");
}

var dtcCount = 0;

function setDTC(code)
{
	if (dtcCount == 6) {
		alert("Up to 6 DTC can bet set. To set additional DTC, please clear previous ones first.");
	}

	if (!sendCommand("ATSET DTC=" + code)) {
	    alert("Unable to set DTC");
	} else {
	    dtcCount++;
	    alert("DTC " + code + " set successfully.\nTotally " + dtcCount + " DTCs has been set");
    }
}

function clearDTC()
{
    if (!sendCommand("ATCLR DTC")) {
        alert("Unable to clear DTC");
    } else {
        alert("DTC cleared successfully");
        dtcCount = 0;
    }
}

function setVIN(vin)
{
    if (!sendCommand("ATSET VIN=" + vin)) {
        alert("Unable to set VIN");
    } else {
        alert("VIN set to " + vin);
    }
}

function setProtocol(protocol)
{
    if (!sendCommand("ATBUS " + protocol)) {
        alert("Unable to set protocol");
    } else {
        alert("Protocol set successfully");
    }
}

function updateFirmware()
{
    httpGet("/navigate?url=FirmwareUpdater.exe&arg=" + portConnected);
    disconnect();
}

function replayLogFile()
{
	alert("This feature is not implemented yet");
}

function setDefaultPidValue(index) {
    if (index < pidDefaults.length && portConnected) {
        var pid = pidDefaults[index].getAttribute("id");
        var value = pidDefaults[index].getAttribute("value");
        if (!sendCommand("ATSET " + pid + "=" + value)) {
            alert("Error setting PID " + pid);
            return;
        }
        for (m = 0; m < groups.length; m++) {
            var e = document.getElementById(groups[m].getAttribute("name") + "-" + pid);
            if (e) {
                e.value = value;
            }
        }
        self.setTimeout("setDefaultPidValue(" + (index + 1) + ")", 1);
    } else {
		emulatorConnected = true;
	}
}

function SetPidValue(pid, value)
{
    if (!SerialWrite("ATSET " + pid + "=" + value + "\r")) {
        alert("Emulator not connected");
        return false;
    }
	SerialRead();
    return true;
}

function toggleIgnition()
{
    if (!sendCommand(ignition ? "ATACC0" : "ATACC1")) {
        alert("Unable to set ignition status");
        return false;
    }
	ignition = !ignition;
	var e = document.getElementById("button_ignite");
	e.innerHTML = "Ignition is " + (ignition ? "ON" : "OFF");
	e.className = ignition ? "ignite_button" : "ignite_button_off";
    return true;
}

var pidElement;
var pidDiff;
var pidInterval;
var pidValueMin;
var pidValueMax;
var pidActive;

function StartChangeValue(element, pid, diff, minval, maxval)
{
    pidActive = pid;
    pidElement = element;
	pidDiff = diff;
	pidInterval = 200;
	pidValueMin = minval;
	pidValueMax = maxval;
	timerChangeValue();
}

function timerChangeValue()
{
	if (pidElement) {
		var val = parseInt(pidElement.value);
		val += pidDiff;
		if (val < pidValueMin)
		    val = pidValueMin;
		else if (val > pidValueMax)
		    val = pidValueMax;
		
		if (SetPidValue(pidActive, val) && pidDiff != 0) {
   		    pidElement.value = val;
   		    if (pidInterval > 10)
   		        pidInterval -= 10;
   		    else if (pidDiff > 0)
   		        pidDiff++;
   		    else
   		        pidDiff--;
   		    if (pidInterval) self.setTimeout(timerChangeValue, pidInterval);
		}
	}
}

function StopChangeValue()
{
    pidDiff = 0;
    pidInterval = 0;
    timerChangeValue();
    pidElement = null;
}

var pidDisabled = new Array(256);

function togglePID(pid, enabled)
{
	var i = parseInt(pid, 16) & 0xff;
	pidDisabled[i] = !enabled;
	var specialPid = i & 0xE0;
	var val = 0;
	var vals = "";
	var pidTmp = specialPid + 1;
	for (m = 0; m < 4; m++) {
		val = 0;
		for (n = 1; n <= 8; n++) {
			val <<= 1;
			if (!pidDisabled[pidTmp++]) val |= 1;
		}
		vals += val.toString(16) + ",";
	}
	specialPid |= 0x100;
	var cmd = "ATSET 0" + specialPid.toString(16) + "=" + vals.substr(0, vals.length - 1);
	alert(cmd);
    if (!sendCommand(cmd)) {
        alert("Unable to set PID " + specialPid.toString(16));
        return false;
    }

}